@@ -1,232 +1,194 @@
 """The core UML metamodel events."""
 
-from interfaces import *
-from zope import interface
+from gaphor.event import ServiceEvent
 
-class AttributeChangeEvent(object):
+
+class ElementUpdated:
+    """
+    Generic event fired when element state changes.
+    """
+
+    def __init__(self, element, property):
+        self.element = element
+        self.property = property
+
+
+class AttributeUpdated(ElementUpdated):
     """A UML attribute has changed value."""
-    
-    interface.implements(IAttributeChangeEvent)
 
     def __init__(self, element, attribute, old_value, new_value):
         """Constructor.  The element parameter is the element with the
         changing attribute.  The attribute parameter is the parameter
         element that changed.  The old_value is the old value of the attribute
         and the new_value is the new value of the attribute."""
-        
-        self.element = element
-        self.property = attribute
+
+        super().__init__(element, attribute)
         self.old_value = old_value
         self.new_value = new_value
 
 
-class AssociationChangeEvent(object):
+class AssociationUpdated(ElementUpdated):
     """An association UML element has changed."""
-    
-    interface.implements(IAssociationChangeEvent)
 
     def __init__(self, element, association):
         """Constructor.  The element parameter is the element the association
         is changing from.  The association parameter is the changed
         association element."""
-        
-        self.element = element
-        self.property = association
+
+        super().__init__(element, association)
 
 
-class AssociationSetEvent(AssociationChangeEvent):
+class AssociationSet(AssociationUpdated):
     """An association element has been set."""
-    
-    interface.implements(IAssociationSetEvent)
 
     def __init__(self, element, association, old_value, new_value):
         """Constructor.  The element parameter is the element setting the
         association element.  The association parameter is the association
         element being set.  The old_value parameter is the old association
         and the new_value parameter is the new association."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
+
+        super().__init__(element, association)
         self.old_value = old_value
         self.new_value = new_value
 
 
-class AssociationAddEvent(AssociationChangeEvent):
+class AssociationAdded(AssociationUpdated):
     """An association element has been added."""
 
-    interface.implements(IAssociationAddEvent)
-
     def __init__(self, element, association, new_value):
         """Constructor.  The element parameter is the element the association
         has been added to.  The association parameter is the association
         element being added."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
+
+        super().__init__(element, association)
         self.new_value = new_value
 
 
-class AssociationDeleteEvent(AssociationChangeEvent):
+class AssociationDeleted(AssociationUpdated):
     """An association element has been deleted."""
 
-    interface.implements(IAssociationDeleteEvent)
-
     def __init__(self, element, association, old_value):
         """Constructor.  The element parameter is the element the association
         has been deleted from.  The association parameter is the deleted
         association element."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
+
+        super().__init__(element, association)
         self.old_value = old_value
 
 
-class DerivedChangeEvent(AssociationChangeEvent):
+class DerivedUpdated(AssociationUpdated):
     """A derived property has changed."""
-    pass
 
-class DerivedSetEvent(DerivedChangeEvent):
+
+class DerivedSet(AssociationSet, DerivedUpdated):
     """A generic derived set event."""
 
-    interface.implements(IAssociationSetEvent)
-
     def __init__(self, element, association, old_value, new_value):
         """Constructor.  The element parameter is the element to which the
         derived set belongs.  The association parameter is the association
         of the derived set."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.old_value = old_value
-        self.new_value = new_value
+
+        super().__init__(element, association, old_value, new_value)
 
 
-class DerivedAddEvent(DerivedChangeEvent):
+class DerivedAdded(AssociationAdded, DerivedUpdated):
     """A derived property has been added."""
 
-    interface.implements(IAssociationAddEvent)
-
     def __init__(self, element, association, new_value):
         """Constructor.  The element parameter is the element to which the
-        derived property belongs.  The association parameter is the 
+        derived property belongs.  The association parameter is the
         association of the derived property."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.new_value = new_value
+
+        super().__init__(element, association, new_value)
 
 
-class DerivedDeleteEvent(DerivedChangeEvent):
+class DerivedDeleted(AssociationDeleted, DerivedUpdated):
     """A derived property has been deleted."""
 
-    interface.implements(IAssociationDeleteEvent)
-
     def __init__(self, element, association, old_value):
         """Constructor.  The element parameter is the element to which the
-        derived property belongs.  The association parameter is the 
+        derived property belongs.  The association parameter is the
         association of the derived property."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.old_value = old_value
+
+        super().__init__(element, association, old_value)
 
 
-class RedefineSetEvent(AssociationChangeEvent):
+class RedefinedSet(AssociationSet):
     """A redefined property has been set."""
 
-    interface.implements(IAssociationSetEvent)
-
     def __init__(self, element, association, old_value, new_value):
         """Constructor.  The element parameter is the element to which the
-        property belongs.  The association parameter is association of the 
+        property belongs.  The association parameter is association of the
         property."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.old_value = old_value
-        self.new_value = new_value
+
+        super().__init__(element, association, old_value, new_value)
 
 
-class RedefineAddEvent(AssociationChangeEvent):
+class RedefinedAdded(AssociationAdded):
     """A redefined property has been added."""
 
-    interface.implements(IAssociationAddEvent)
-
     def __init__(self, element, association, new_value):
         """Constructor.  The element parameter is the element to which the
         property belongs.  The association parameter is the association of
         the property."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.new_value = new_value
+
+        super().__init__(element, association, new_value)
 
 
-class RedefineDeleteEvent(AssociationChangeEvent):
+class RedefinedDeleted(AssociationDeleted):
     """A redefined property has been deleted."""
-    
-    interface.implements(IAssociationDeleteEvent)
 
     def __init__(self, element, association, old_value):
         """Constructor.  The element parameter is the element to which the
         property belongs.  The association parameter is the association of
         the property."""
-        
-        AssociationChangeEvent.__init__(self, element, association)
-        self.old_value = old_value
+
+        super().__init__(element, association, old_value)
 
 
-class DiagramItemCreateEvent(object):
-    """A diagram item has been created."""
-    
-    interface.implements(IElementCreateEvent)
-
-    def __init__(self, element):
-        """Constructor.  The element parameter is the element being created."""
-        
-        self.element = element
-
-
-class ElementCreateEvent(object):
+class ElementCreated(ServiceEvent):
     """An element has been created."""
-    
-    interface.implements(IElementCreateEvent, IElementFactoryEvent)
 
     def __init__(self, service, element):
         """Constructor.  The service parameter is the service responsible
         for creating the element.  The element parameter is the element being
         created."""
-        
-        self.service = service
+        super().__init__(service)
         self.element = element
 
 
-class ElementDeleteEvent(object):
+class DiagramItemCreated(ElementCreated):
+    """A diagram item has been created."""
+
+    def __init__(self, service, element):
+        """Constructor.  The element parameter is the element being created."""
+        super().__init__(service, element)
+
+
+class ElementDeleted(ServiceEvent):
     """An element has been deleted."""
-    
-    interface.implements(IElementDeleteEvent, IElementFactoryEvent)
 
     def __init__(self, service, element):
         """Constructor.  The service parameter is the service responsible for
         deleting the element.  The element parameter is the element being
         deleted."""
-        
-        self.service = service
+        super().__init__(service)
         self.element = element
 
 
-class ModelFactoryEvent(object):
+class ModelReady(ServiceEvent):
     """A generic element factory event."""
-    
-    interface.implements(IModelFactoryEvent)
 
     def __init__(self, service):
         """Constructor.  The service parameter is the service the emitted the
         event."""
-        
-        self.service = service
+        super().__init__(service)
 
 
-class FlushFactoryEvent(object):
+class ModelFlushed(ServiceEvent):
     """The element factory has been flushed."""
-    
-    interface.implements(IFlushFactoryEvent)
 
     def __init__(self, service):
         """Constructor.  The service parameter is the service responsible for
         flushing the factory."""
-        
-        self.service = service
-
+        super().__init__(service)
