@@ -9,12 +9,32 @@ Functions collected in this module allow to
 """
 
 import itertools
-from gaphor.UML.uml2 import *
+from typing import Iterable, Sequence
 
-# '<<%s>>'
-STEREOTYPE_FMT = '\xc2\xab%s\xc2\xbb'
+from gaphor.UML.uml2 import (
+    Association,
+    Class,
+    Classifier,
+    Component,
+    Dependency,
+    Element,
+    Extension,
+    ExtensionEnd,
+    Generalization,
+    Implementation,
+    InstanceSpecification,
+    Interface,
+    Message,
+    MessageOccurrenceSpecification,
+    Property,
+    Realization,
+    Slot,
+    Stereotype,
+    Usage,
+)
 
-def stereotypes_str(element, stereotypes=()):
+
+def stereotypes_str(element: Element, stereotypes: Sequence[str] = ()):
     """
     Identify stereotypes of an UML metamodel instance and return coma
     separated stereotypes as string.
@@ -27,14 +47,16 @@ def stereotypes_str(element, stereotypes=()):
     """
     # generate string with stereotype names separated by coma
     if element:
-        applied = (stereotype_name(st) for st in get_applied_stereotypes(element))
+        applied: Iterable[str] = (
+            stereotype_name(st) for st in get_applied_stereotypes(element)
+        )
     else:
         applied = ()
-    s = ', '.join(itertools.chain(stereotypes, applied))
+    s = ", ".join(itertools.chain(stereotypes, applied))
     if s:
-        return STEREOTYPE_FMT % s
+        return f"«{s}»"
     else:
-        return ''
+        return ""
 
 
 def stereotype_name(stereotype):
@@ -48,37 +70,43 @@ def stereotype_name(stereotype):
     """
     name = stereotype.name
     if not name:
-        return ''
+        return ""
     elif len(name) > 1 and name[1].isupper():
         return name
     else:
         return name[0].lower() + name[1:]
 
 
-def apply_stereotype(factory, element, stereotype):
+def apply_stereotype(element, stereotype):
     """
     Apply a stereotype to an element.
 
     :Parameters:
-     factory
-        UML metamodel factory.
      element
         UML metamodel class instance.
      stereotype
         UML metamodel stereotype instance.
     """
-    obj = factory.create(InstanceSpecification)
+    assert (
+        element.model is stereotype.model
+    ), "Element and Stereotype are from different models"
+    model = element.model
+    obj = model.create(InstanceSpecification)
     obj.classifier = stereotype
     element.appliedStereotype = obj
     return obj
 
 
-def find_instances(factory, element):
+def find_instances(element):
     """
     Find instance specification which extend classifier `element`.
     """
-    return factory.select(lambda e: e.isKindOf(InstanceSpecification) \
-            and e.classifier and e.classifier[0] == element)
+    model = element.model
+    return model.select(
+        lambda e: e.isKindOf(InstanceSpecification)
+        and e.classifier
+        and e.classifier[0] == element
+    )
 
 
 def remove_stereotype(element, stereotype):
@@ -98,10 +126,11 @@ def remove_stereotype(element, stereotype):
             break
 
 
-def get_stereotypes(factory, element):
+def get_stereotypes(element):
     """
     Get sorted collection of possible stereotypes for specified element.
     """
+    model = element.model
     # UML specs does not allow to extend stereotypes with stereotypes
     if isinstance(element, Stereotype):
         return ()
@@ -109,12 +138,12 @@ def get_stereotypes(factory, element):
     cls = type(element)
 
     # find out names of classes, which are superclasses of element class
-    names = set(c.__name__ for c in cls.__mro__ if issubclass(c, Element))
+    names = {c.__name__ for c in cls.__mro__ if issubclass(c, Element)}
 
     # find stereotypes that extend element class
-    classes = factory.select(lambda e: e.isKindOf(Class) and e.name in names)
-    
-    stereotypes = set(ext.ownedEnd.type for cls in classes for ext in cls.extension)
+    classes = model.select(lambda e: e.isKindOf(Class) and e.name in names)
+
+    stereotypes = {ext.ownedEnd.type for cls in classes for ext in cls.extension}
     return sorted(stereotypes, key=lambda st: st.name)
 
 
@@ -125,68 +154,107 @@ def get_applied_stereotypes(element):
     return element.appliedStereotype[:].classifier
 
 
-def create_extension(factory, element, stereotype):
+def create_extension(metaclass: Class, stereotype: Stereotype) -> Extension:
     """
-    Extend an element with a stereotype.
+    Create an Extension association between an metaclass and a stereotype.
     """
-    ext = factory.create(Extension)
-    p = factory.create(Property)
-    ext_end = factory.create(ExtensionEnd)
+    assert (
+        metaclass.model is stereotype.model
+    ), "Metaclass and Stereotype are from different models"
+
+    model = metaclass.model
+    ext: Extension = model.create(Extension)
+    p = model.create(Property)
+    ext_end = model.create(ExtensionEnd)
 
     ext.memberEnd = p
     ext.memberEnd = ext_end
     ext.ownedEnd = ext_end
     ext_end.type = stereotype
-    ext_end.aggregation = 'composite'
-    p.type = element
-    p.name = 'baseClass'
+    ext_end.aggregation = "composite"
+    p.type = metaclass
+    p.name = "baseClass"
     stereotype.ownedAttribute = p
+    metaclass.ownedAttribute = ext_end
 
     return ext
 
-extend_with_stereotype = create_extension
 
-def add_slot(factory, instance, definingFeature):
+def is_metaclass(element):
+    return (
+        (not isinstance(element, Stereotype))
+        and hasattr(element, "extension")
+        and bool(element.extension)
+    )
+
+
+def add_slot(instance, definingFeature):
     """
     Add slot to instance specification for an attribute.
     """
-    slot = factory.create(Slot)
+    assert (
+        instance.model is definingFeature.model
+    ), "Instance and Defining feature are from different models"
+    model = instance.model
+    slot = model.create(Slot)
     slot.definingFeature = definingFeature
     instance.slot = slot
     return slot
 
 
-def create_dependency(factory, supplier, client):
-    dep = factory.create(Dependency)
+def create_dependency(supplier, client):
+    assert (
+        supplier.model is client.model
+    ), "Supplier and Client are from different models"
+    model = supplier.model
+    dep = model.create(Dependency)
     dep.supplier = supplier
     dep.client = client
     return dep
 
-def create_realization(factory, realizingClassifier, abstraction):
-    dep = factory.create(Realization)
+
+def create_realization(realizingClassifier, abstraction):
+    assert (
+        realizingClassifier.model is abstraction.model
+    ), "Realizing classifier and Abstraction are from different models"
+    model = realizingClassifier.model
+    dep = model.create(Realization)
     dep.realizingClassifier = realizingClassifier
     dep.abstraction = abstraction
     return dep
 
-def create_generalization(factory, general, specific):
-    gen = factory.create(Generalization)
+
+def create_generalization(general, specific):
+    assert (
+        general.model is specific.model
+    ), "General and Specific are from different models"
+    model = general.model
+    gen = model.create(Generalization)
     gen.general = general
     gen.specific = specific
     return gen
 
-def create_implementation(factory, contract, implementatingClassifier):
-    impl = factory.create(Implementation)
+
+def create_implementation(contract, implementatingClassifier):
+    assert (
+        contract.model is implementatingClassifier.model
+    ), "Contract and Implementating classifier are from different models"
+    model = contract.model
+    impl = model.create(Implementation)
     impl.contract = contract
     impl.implementatingClassifier = implementatingClassifier
     return impl
 
-def create_association(factory, type_a, type_b):
+
+def create_association(type_a, type_b):
     """
     Create an association between two items.
     """
-    assoc = factory.create(Association)
-    end_a = factory.create(Property)
-    end_b = factory.create(Property)
+    assert type_a.model is type_b.model, "Head and Tail end are from different models"
+    model = type_a.model
+    assoc = model.create(Association)
+    end_a = model.create(Property)
+    end_b = model.create(Property)
     assoc.memberEnd = end_a
     assoc.memberEnd = end_b
     end_a.type = type_a
@@ -201,7 +269,7 @@ def set_navigability(assoc, end, nav):
     """
     Set navigability of an association end (property).
 
-    There are free possible values for ``nav`` parameter
+    There are three possible values for ``nav`` parameter
 
      True
         association end is navigable
@@ -210,7 +278,7 @@ def set_navigability(assoc, end, nav):
      None
         association end navigability is unkown
 
-    There are two ways of specifing than an end is navigable
+    There are two ways of specifing that an end is navigable
 
     - an end is in Association.navigableOwnedEnd collection
     - an end is class (interface) attribute (stored in Class.ownedAttribute
@@ -280,7 +348,7 @@ def dependency_type(client, supplier):
     If none of above is detected then standard dependency is determined.
     """
     dt = Dependency
-        
+
     # test interface first as it is a classifier
     if isinstance(supplier, Interface):
         dt = Usage
@@ -290,24 +358,23 @@ def dependency_type(client, supplier):
     return dt
 
 
-def create_message(factory, msg, inverted=False):
+def clone_message(msg, inverted=False):
     """
     Create new message based on speciied message.
 
     If inverted is set to True, then inverted message is created.
     """
-    message = factory.create(Message)
+    model = msg.model
+    message = model.create(Message)
     send = None
     receive = None
 
     if msg.sendEvent:
-        send = factory.create(MessageOccurrenceSpecification)
-        sl = msg.sendEvent.covered
-        send.covered = sl
+        send = model.create(MessageOccurrenceSpecification)
+        send.covered = msg.sendEvent.covered
     if msg.receiveEvent:
-        receive = factory.create(MessageOccurrenceSpecification)
-        rl = msg.receiveEvent.covered
-        receive.covered = rl
+        receive = model.create(MessageOccurrenceSpecification)
+        receive.covered = msg.receiveEvent.covered
 
     if inverted:
         # inverted message goes in different direction, than original
@@ -320,4 +387,12 @@ def create_message(factory, msg, inverted=False):
     return message
 
 
-#vim:sw=4:et:ai
+def swap_element(element, new_class):
+    """
+    A "trick" to swap the element type.
+
+    Used in certain cases where the underlaying element type
+    may change.
+    """
+    if element.__class__ is not new_class:
+        element.__class__ = new_class
