@@ -6,8 +6,8 @@ Pseudostates
 There are some similarities between activities and state machines, for example
     - initial node and initial psuedostate
     - final node and final state
-Of course, they differ in many aspects, but the similarites drive user
-interface of state machines. This is with respect of minimalization of the set
+Of course, they differ in many aspects, but the similarities drive user
+interface of state machines. This is with respect of minimization of the set
 of diagram items (i.e. there is only one diagram item for both join and fork
 nodes in activities implemented in Gaphor).
 
@@ -19,12 +19,17 @@ There are separate diagram items for pseudostates
     we start to implement more pseudostates].
 """
 
-from gaphor.diagram.nameditem import NamedItem
+from gaphor.diagram.states.finalstate import FinalStateItem
+from gaphor.diagram.states.pseudostates import (
+    HistoryPseudostateItem,
+    InitialPseudostateItem,
+)
+from gaphor.diagram.states.state import StateItem
+from gaphor.diagram.states.transition import TransitionItem
 
-class VertexItem(NamedItem):
-    """
-    Abstract class for all vertices. All state, pseudostate items derive
-    from VertexItem, which simplifies transition connection adapters.
-    """
-    pass
 
+def _load():
+    from gaphor.diagram.states import propertypages, vertexconnect
+
+
+_load()
