@@ -30,78 +30,95 @@ The generator parse_generator(filename, loader) may be used if the loading
 takes a long time. The yielded values are the percentage of the file read.
 """
 
-__all__ = [ 'parse', 'ParserException' ]
+from __future__ import annotations
 
+import io
+import logging
 import os
-import types
+from collections import OrderedDict
+from typing import IO, Dict, List, Optional, Tuple, Union
 from xml.sax import handler
-from cStringIO import InputType
 
-from gaphor.misc.odict import odict
+__all__ = ["parse", "ParserException"]
 
-class base(object):
+log = logging.getLogger(__name__)
+
+
+class base:
     """Simple base class for element, canvas and canvasitem.
     """
 
     def __init__(self):
-        self.values = { }
-        self.references = { }
+        self.values: Dict[str, str] = {}
+        self.references: Dict[str, Union[str, List[str]]] = {}
 
     def __getattr__(self, key):
-        return self[key]
+        try:
+            return self.__getitem__(key)
+        except KeyError as e:
+            raise AttributeError(e)
 
     def __getitem__(self, key):
         try:
             return self.values[key]
-        except:
+        except KeyError:
             return self.references[key]
 
     def get(self, key):
         try:
             return self.__getitem__(key)
-        except:
+        except KeyError:
             return None
 
-class element(base):
 
-    def __init__(self, id, type):
+class element(base):
+    def __init__(self, id: str, type: str, canvas: Optional[canvas] = None):
         base.__init__(self)
         self.id = id
         self.type = type
-        self.canvas = None
+        self.canvas = canvas
+        self.element: object = None
+
 
 class canvas(base):
-
-    def __init__(self):
+    def __init__(self, canvasitems: Optional[List[canvasitem]] = None):
         base.__init__(self)
-        self.canvasitems = []
+        self.canvasitems: List[canvasitem] = canvasitems or []
+
 
 class canvasitem(base):
-
-    def __init__(self, id, type):
+    def __init__(
+        self, id: str, type: str, canvasitems: Optional[List[canvasitem]] = None
+    ):
         base.__init__(self)
         self.id = id
         self.type = type
-        self.canvasitems = []
+        self.canvasitems: List[canvasitem] = canvasitems or []
 
 
-XMLNS='http://gaphor.sourceforge.net/model'
+XMLNS = "http://gaphor.sourceforge.net/model"
+
 
 class ParserException(Exception):
     pass
 
+
 # Loader state:
-[ ROOT,         # Expect 'gaphor' element
-  GAPHOR,       # Expect UML classes (tag name is the UML class name)
-  ELEMENT,      # Expect properties of UML object
-  DIAGRAM,      # Expect properties of Diagram object + canvas
-  CANVAS,       # Expect canvas properties + <item> tags
-  ITEM,         # Expect item attributes and nested items
-  ATTR,         # Reading contents of an attribute (such as a <val> or <ref>)
-  VAL,          # Redaing contents of a <val> tag
-  REFLIST,      # In a <reflist>
-  REF           # Reading contents of a <ref> tag
-] = xrange(10)
+[
+    ROOT,  # Expect 'gaphor' element
+    GAPHOR,  # Expect UML classes (tag name is the UML class name)
+    ELEMENT,  # Expect properties of UML object
+    DIAGRAM,  # Expect properties of Diagram object + canvas
+    CANVAS,  # Expect canvas properties + <item> tags
+    ITEM,  # Expect item attributes and nested items
+    ATTR,  # Reading contents of an attribute (such as a <val> or <ref>)
+    VAL,  # Redaing contents of a <val> tag
+    REFLIST,  # In a <reflist>
+    REF,  # Reading contents of a <ref> tag
+] = range(10)
+
+State = int
+
 
 class GaphorLoader(handler.ContentHandler):
     """Create a list of elements. an element may contain a canvas and a
@@ -117,115 +134,143 @@ class GaphorLoader(handler.ContentHandler):
     def push(self, element, state):
         """Add an element to the item stack.
         """
-        self.__stack.append((element, state))
+        self._stack.append((element, state))
 
     def pop(self):
         """Return the last item on the stack. The item is removed from
         the stack.
         """
-        return self.__stack.pop()[0]
+        return self._stack.pop()[0]
 
     def peek(self, depth=1):
         """Return the last item on the stack. The item is not removed.
         """
-        return self.__stack[-1 * depth][0]
+        return self._stack[-1 * depth][0]
 
     def state(self):
         """Return the current state of the parser.
         """
         try:
-            return self.__stack[-1][1]
+            return self._stack[-1][1]
         except IndexError:
             return ROOT
 
-    def endDTD(self):
-        pass
-
     def startDocument(self):
         """Start of document: all our attributes are initialized.
         """
         self.version = None
         self.gaphor_version = None
-        self.elements = odict() # map id: element/canvasitem
-        self.__stack = []
-        self.text = ''
+        self.elements: Dict[str, Union[element, canvasitem]] = OrderedDict()
+        self._stack: List[Tuple[Union[element, canvas, canvasitem], State]] = []
+        self.text = ""
+        self._start_element_handlers = (
+            self.start_element,
+            self.start_canvas,
+            self.start_canvas_item,
+            self.start_attribute,
+            self.start_reference,
+            self.start_attribute_value,
+            self.start_root,
+            self.invalid_tag,
+        )
 
     def endDocument(self):
-        if len(self.__stack) != 0:
-            raise ParserException, 'Invalid XML document.'
+        if len(self._stack) != 0:
+            raise ParserException("Invalid XML document.")
 
     def startElement(self, name, attrs):
-        self.text = ''
-        
+        self.text = ""
+
         state = self.state()
 
-        # Read a element class. The name of the tag is the class name:
-        if state == GAPHOR:
-            id = attrs['id']
-            e = element(id, name)
-            assert id not in self.elements.keys(), '%s already defined' % (id)#, self.elements[id])
-            self.elements[id] = e
-            self.push(e, name == 'Diagram' and DIAGRAM or ELEMENT)
+        for h in self._start_element_handlers:
+            if h(state, name, attrs):
+                break
 
+    def start_element(self, state, name, attrs):
+        # Read an element class. The name of the tag is the class name:
+        if state == GAPHOR:
+            id = attrs["id"]
+            e = element(id, name)
+            if id in self.elements.keys():
+                log.exception(f"File corrupted, remove element {id} and try again")
+            self.elements[id] = e
+            self.push(e, name == "Diagram" and DIAGRAM or ELEMENT)
+            return True
+
+    def start_canvas(self, state, name, attrs):
         # Special treatment for the <canvas> tag in a Diagram:
-        elif state == DIAGRAM and name == 'canvas':
+        if state == DIAGRAM and name == "canvas":
             c = canvas()
             self.peek().canvas = c
             self.push(c, CANVAS)
+            return True
 
+    def start_canvas_item(self, state, name, attrs):
         # Items in a canvas are referenced by the <item> tag:
-        elif state in (CANVAS, ITEM) and name == 'item':
-            id = attrs['id']
-            c = canvasitem(id, attrs['type'])
-            assert id not in self.elements.keys(), '%s already defined' % id
-            self.elements[id] = c
-            self.peek().canvasitems.append(c)
-            self.push(c, ITEM)
+        if state in (CANVAS, ITEM) and name == "item":
+            id = attrs["id"]
+            ci = canvasitem(id, attrs["type"])
+            assert id not in self.elements.keys(), f"{id} already defined"
+            self.elements[id] = ci
+            self.peek().canvasitems.append(ci)
+            self.push(ci, ITEM)
+            return True
 
+    def start_attribute(self, state, name, attrs):
         # Store the attribute name on the stack, so we can use it later
         # to store the <ref>, <reflist> or <val> content:
-        elif state in (ELEMENT, DIAGRAM, CANVAS, ITEM):
+        if state in (ELEMENT, DIAGRAM, CANVAS, ITEM):
             # handle 'normal' attributes
             self.push(name, ATTR)
+            return True
 
+    def start_reference(self, state, name, attrs):
         # Reference list:
-        elif state == ATTR and name == 'reflist':
+        if state == ATTR and name == "reflist":
             self.push(self.peek(), REFLIST)
+            return True
 
         # Reference with multiplicity 1:
-        elif state  == ATTR and name == 'ref':
+        elif state == ATTR and name == "ref":
             n = self.peek(1)
             # Fetch the element instance from the stack
-            r = self.peek(2).references[n] = attrs['refid']
+            r = self.peek(2).references[n] = attrs["refid"]
             self.push(None, REF)
+            return True
 
         # Reference with multiplicity *:
-        elif state == REFLIST and name == 'ref':
+        elif state == REFLIST and name == "ref":
             n = self.peek(1)
             # Fetch the element instance from the stack
             r = self.peek(3).references
-            refid = attrs['refid']
+            refid = attrs["refid"]
             try:
                 r[n].append(refid)
             except KeyError:
                 r[n] = [refid]
             self.push(None, REF)
+            return True
 
+    def start_attribute_value(self, state, name, attrs):
         # We need to get the text within the <val> tag:
-        elif state == ATTR and name == 'val':
+        if state == ATTR and name == "val":
             self.push(None, VAL)
+            return True
 
+    def start_root(self, state, name, attrs):
         # The <gaphor> tag is the toplevel tag:
-        elif state == ROOT and name == 'gaphor':
-            assert attrs['version'] in ('3.0',)
-            self.version = attrs['version']
-            self.gaphor_version = attrs.get('gaphor-version')
+        if state == ROOT and name == "gaphor":
+            assert attrs["version"] in ("3.0",)
+            self.version = attrs["version"]
+            self.gaphor_version = attrs.get("gaphor-version")
             if not self.gaphor_version:
-                self.gaphor_version = attrs.get('gaphor_version')
+                self.gaphor_version = attrs.get("gaphor_version")
             self.push(None, GAPHOR)
+            return True
 
-        else:
-            raise ParserException, 'Invalid XML: tag <%s> not known (state = %s)' % (name, state)
+    def invalid_tag(self, state, name, attrs):
+        raise ParserException(f"Invalid XML: tag <{name}> not known (state = {state})")
 
     def endElement(self, name):
         # Put the text on the value
@@ -238,8 +283,8 @@ class GaphorLoader(handler.ContentHandler):
 
     def startElementNS(self, name, qname, attrs):
         if not name[0] or name[0] == XMLNS:
-            a = { }
-            for key, val in attrs.items():
+            a = {}
+            for key, val in list(attrs.items()):
                 a[key[1]] = val
             self.startElement(name[1], a)
 
@@ -266,47 +311,48 @@ def parse_generator(filename, loader):
     """The generator based version of parse().
     parses the file filename and load it with ContentHandler loader.
     """
-    assert isinstance(loader, GaphorLoader), 'loader should be a GaphorLoader'
+    assert isinstance(loader, GaphorLoader), "loader should be a GaphorLoader"
     from xml.sax import make_parser
+
     parser = make_parser()
 
     parser.setFeature(handler.feature_namespaces, 1)
     parser.setContentHandler(loader)
 
-    for percentage in parse_file(filename, parser):
-        yield percentage
+    yield from parse_file(filename, parser)
 
 
-class ProgressGenerator(object):
-    """A generator that yields the progress of taking from a file input object 
+class ProgressGenerator:
+    """A generator that yields the progress of taking from a file input object
     and feeding it into an output object.  The supplied file object is neither
     opened not closed by this generator.  The file object is assumed to
     already be opened for reading and that it will be closed elsewhere."""
-    
+
     def __init__(self, input, output, block_size=512):
         """Initialize the progress generator.  The input parameter is a file
-        object.  The ouput parameter is usually a SAX parser but can be 
+        object.  The output parameter is usually a SAX parser but can be
         anything that implements a feed() method.  The block size is the size
         of each block that is read from the input."""
-        
+
         self.input = input
         self.output = output
         self.block_size = block_size
-        if isinstance(self.input, types.FileType):
-            self.file_size = os.fstat(self.input.fileno())[6]
-        elif isinstance(self.input, InputType):
-            self.file_size = len(self.input.getvalue())
-            self.input.reset()
-                
+        if isinstance(self.input, io.IOBase):
+            orig_pos = self.input.tell()
+            self.file_size = self.input.seek(0, 2)
+            self.input.seek(orig_pos, os.SEEK_SET)
+        elif isinstance(self.input, str):
+            self.file_size = len(self.input)
+
     def __iter__(self):
         """Return a generator that yields the progress of reading data
         from the input and feeding it into the output.  The progress
         yielded in each iteration is the percentage of data read, relative
         to the to input file size."""
-        
+
         block = self.input.read(self.block_size)
         read_size = len(block)
-        
+
         while block:
             self.output.feed(block)
             block = self.input.read(self.block_size)
@@ -319,21 +365,18 @@ def parse_file(filename, parser):
     should be a GaphorLoader instance.  The filename parameter can be an
     open file descriptor instance or the name of a file.  The progress
     percentage of the parser is yielded."""
-    
+
     is_fd = True
-    
-    if isinstance(filename, (types.FileType, InputType)):
-        file_obj = filename
+
+    if isinstance(filename, io.IOBase):
+        file_obj: Union[IO, io.IOBase] = filename
     else:
         is_fd = False
-        file_obj = open(filename, 'rb')
-        
-    for progress in ProgressGenerator(file_obj, parser):
-        yield progress
-    
+        file_obj = open(filename, "r")
+
+    yield from ProgressGenerator(file_obj, parser)
+
     parser.close()
-    
+
     if not is_fd:
         file_obj.close()
-
-# vim:sw=4:et:ai
