@@ -1,53 +1,57 @@
 """
-Transation support for Gaphor
+Transaction support for Gaphor
 """
 
-from logging import getLogger
-from zope import interface, component
-from gaphor.interfaces import ITransaction
+import logging
+from typing import Callable, List, Set
+
 from gaphor.event import TransactionBegin, TransactionCommit, TransactionRollback
-from gaphor import application
 
-logger = getLogger('transaction')
+log = logging.getLogger(__name__)
+
+# Sessions should subscribe a hook to receive events emited via @transactional
+subscribers: Set[Callable[[object], None]] = set()
+
 
 def transactional(func):
-    """The transactional decorator makes a function transactional.  A
-    Transaction instance is created before the decorated function is called.
-    If calling the function leads to an exception being raised, the transaction
-    is rolled-back.  Otherwise, it is committed."""
-    
+    """
+    The transactional decorator makes a function transactional.
+    Events are emitted through the (global) `subscribers` set.
+
+    It is preferred to use the `Transaction` context manager. The context manager
+    emits events in the context of the session in scope, whereas the `@transactional`
+    decorator emits a global event which is sent to the active session.
+    """
+
     def _transactional(*args, **kwargs):
-        r = None
-        tx = Transaction()
-        try:
-            r = func(*args, **kwargs)
-        except Exception, e:
-            log.error('Transaction terminated due to an exception, performing a rollback', exc_info=True)
-            try:
-                tx.rollback()
-            except Exception, e:
-                log.error('Rollback failed', exc_info=True)
-            raise
-        else:
-            tx.commit()
-        return r
+        if __debug__ and args and hasattr(args[0], "event_manager"):
+            log.warning(f"Consider using the Transaction context manager for {args[0]}")
+
+        with Transaction(_SubscribersHandler):
+            return func(*args, **kwargs)
+
     return _transactional
 
+
 class TransactionError(Exception):
     """
     Errors related to the transaction module.
     """
 
-class Transaction(object):
+
+class Transaction:
     """
-    The transaction. On start and end of a transaction an event is emited.
+    The transaction. On start and end of a transaction an event is emitted.
+
+    >>> import gaphor.services.eventmanager
+    >>> event_manager = gaphor.services.eventmanager.EventManager()
 
     Transactions can be nested. If the outermost transaction is committed or
     rolled back, an event is emitted.
 
     Events can be handled programmatically:
 
-    >>> tx = Transaction()
+    >>> tx = Transaction(event_manager)
     >>> tx.commit()
 
     It can be assigned as decorator:
@@ -58,20 +62,17 @@ class Transaction(object):
 
     Or with the ``with`` statement:
 
-    >>> with Transaction():
+    >>> with Transaction(event_manager):
     ...     pass
     """
 
-    interface.implements(ITransaction)
+    _stack: List = []
 
-    component_registry = application.inject('component_registry')
-
-    _stack= []
-
-    def __init__(self):
+    def __init__(self, event_manager):
         """Initialize the transaction.  If this is the first transaction in
-        the stack, a TransactionBegin event is emited."""
-        
+        the stack, a TransactionBegin event is emitted."""
+        self.event_manager = event_manager
+
         self._need_rollback = False
         if not self._stack:
             self._handle(TransactionBegin())
@@ -79,9 +80,9 @@ class Transaction(object):
 
     def commit(self):
         """Commit the transaction.  First, the transaction is closed.
-        If it needs to be rolled-back, a TransactionRollback event is emited.
-        Otherwise, a TransactionCommit event is emited."""
-        
+        If it needs to be rolled-back, a TransactionRollback event is emitted.
+        Otherwise, a TransactionCommit event is emitted."""
+
         self._close()
         if not self._stack:
             if self._need_rollback:
@@ -92,8 +93,8 @@ class Transaction(object):
     def rollback(self):
         """Roll-back the transaction.  First, the transaction is closed.
         Every transaction on the stack is then marked for roll-back.  If
-        the stack is empty, a TransactionRollback event is emited."""
-        
+        the stack is empty, a TransactionRollback event is emitted."""
+
         self._close()
         for tx in self._stack:
             tx._need_rollback = True
@@ -105,34 +106,44 @@ class Transaction(object):
         """Close the transaction.  If the stack is empty, a TransactionError
         is raised.  If the last transaction on the stack isn't this transaction,
         a Transaction error is raised."""
-        
+
         try:
             last = self._stack.pop()
         except IndexError:
-            raise TransactionError, 'No Transaction on stack.'
+            raise TransactionError("No Transaction on stack.")
         if last is not self:
             self._stack.append(last)
-            raise TransactionError, 'Transaction on stack is not the transaction being closed.'
+            raise TransactionError(
+                "Transaction on stack is not the transaction being closed."
+            )
 
     def _handle(self, event):
-        try:
-            component_registry = self.component_registry
-        except (application.NotInitializedError, component.ComponentLookupError):
-            logger.warning('Could not lookup component_registry. Not emiting events.')
-        else:
-            component_registry.handle(event)
+        self.event_manager.handle(event)
 
     def __enter__(self):
         """Provide with-statement transaction support."""
         return self
 
     def __exit__(self, exc_type=None, exc_val=None, exc_tb=None):
-        """Provide with-statement transaction support.  If an error occured,
+        """Provide with-statement transaction support.  If an error occurred,
         the transaction is rolled back.  Otherwise, it is committed."""
-        
+
         if exc_type:
-            self.rollback()
+            log.error(
+                "Transaction terminated due to an exception, performing a rollback",
+                exc_info=True,
+            )
+            try:
+                self.rollback()
+            except Exception:
+                log.error("Rollback failed", exc_info=True)
         else:
             self.commit()
 
-# vim: sw=4:et:ai
+
+class _SubscribersHandler:
+    @classmethod
+    def handle(cls, event):
+        global subscribers
+        for o in subscribers:
+            o(event)
